@'
---
PS1_YAML_FACT1: yaml
PS1_yaml_fact2: 2
ps1_yaml_fact3: |
  one value
  but
  many lines
ps1_yaml_fact4:
  - one
  - two
ps1_yaml_fact5: [first, second, third]
ps1_yaml_fact6:
  red: green
  orange: blue
  yellow: purple
ps1_yaml_fact7: { key: value }
'@
